@@ -1,16 +1,18 @@
-from coldtype import *
-from drafting.geometry import Rect, Edge, Point
+from coldtype.geometry import Rect, Edge, Point
 from coldtype.beziers import CurveCutter, raise_quadratic
 from coldtype.pens.drawablepen import DrawablePenMixin
 from fontTools.pens.basePen import BasePen
+from coldtype.color import Gradient, Color, normalize_color
 
 import math
 import random
 try:
+    # Blender-specific things
     import bpy
     from mathutils import Vector, Matrix
 except:
-    print(">>> Not a blender environment")
+    #print(">>> Not a blender environment")
+    pass
 
 
 class BPH():
@@ -47,7 +49,27 @@ class BPH():
                 bpy.context.scene.collection.children.link(coll)
         return bpy.data.collections.get(name)
 
-    def Primitive(_type, coll, name, dn=False, container=None):
+    def CheckExists(name, dn=False):
+        if dn and name in bpy.context.scene.objects:
+            obj = bpy.context.scene.objects[name]
+            bpy.data.objects.remove(obj, do_unlink=True)
+            return False
+        return name in bpy.context.scene.objects
+    
+    def AddOrFind(name, add_fn, dn=False):
+        if dn and name in bpy.context.scene.objects:
+            obj = bpy.context.scene.objects[name]
+            bpy.data.objects.remove(obj, do_unlink=True)
+            
+        if name not in bpy.context.scene.objects:
+            add_fn()
+            bc = bpy.context.active_object
+            bc.name = name
+            return bc
+        else:
+            return bpy.context.scene.objects[name]
+
+    def Primitive(_type, coll, name, dn=False, container=None, material="auto"):
         created = False
         
         if dn and name in bpy.context.scene.objects:
@@ -60,7 +82,7 @@ class BPH():
                 bpy.ops.curve.primitive_bezier_curve_add()
             elif _type == "Plane":
                 bpy.ops.mesh.primitive_plane_add()
-            bc = bpy.context.active_object
+            bc = bpy.context.object
             bc.name = name
             bc.data.name = name
             if _type == "Bezier":
@@ -75,9 +97,12 @@ class BPH():
                     bc.location[1] = container.y + container.h/2
                     bpy.ops.object.origin_set(type="ORIGIN_CURSOR")
                     bpy.ops.object.transform_apply()
-            mat = bpy.data.materials.new(f"Material_{name}")
-            mat.use_nodes = True
-            bc.data.materials.append(mat)
+            
+            if material:
+                if material == "auto":
+                    mat = bpy.data.materials.new(f"Material_{name}")
+                    mat.use_nodes = True
+                    bc.data.materials.append(mat)
         
         bc = bpy.context.scene.objects[name]
         bc_coll = BPH.FindCollectionForItem(bc)
@@ -97,16 +122,20 @@ class BlenderPen(DrawablePenMixin, BasePen):
     def __init__(self, dat):
         super().__init__(None)
         self.dat = dat
-        tag = self.dat.getTag()
+        tag = self.dat.tag()
+        self.material = None
         self.tag = tag if tag and tag != "Unknown" else f"Curve_{random.randint(0, 1000000)}"
     
     def record(self, dat):
+        self.set_origin(0, 0, 0)
         self._spline = None
         self.splines = []
         self._value = []
         dat.replay(self)
         if self._spline and len(self._spline) > 0 and self._spline not in self.splines:
             self.splines.append(self._spline)
+        x, y = self.dat.ambit().pc
+        #self.set_origin(x/100, y/100, 0)
 
     def _moveTo(self, p):
         if self._spline and len(self._spline) > 0 and self._spline not in self.splines:
@@ -141,7 +170,11 @@ class BlenderPen(DrawablePenMixin, BasePen):
         return self.bez.data.materials
 
     def bsdf(self):
-        return self.materials()[0].node_tree.nodes["Principled BSDF"]
+        if self.material:
+            try:
+                return self.materials()[0].node_tree.nodes["Principled BSDF"]
+            except:
+                return None
     
     def shadow(self, clip=None, radius=10, alpha=0.3, color=Color.from_rgb(0,0,0,1)):
         pass
@@ -159,6 +192,8 @@ class BlenderPen(DrawablePenMixin, BasePen):
             #value[3] = 1
     
     def fill(self, color):
+        if not self.material == "auto" or not self.bsdf():
+            return
         if color:
             if isinstance(color, Gradient):
                 self.fill(color.stops[0][0])
@@ -169,6 +204,8 @@ class BlenderPen(DrawablePenMixin, BasePen):
                 self.setColorValue(dv, color)
     
     def stroke(self, weight=1, color=None, dash=None):
+        if not self.material == "auto" or not self.bsdf():
+            return
         if weight and color and color.a > 0:
             #print("STROKE>>>", self.tag, weight, color)
             self.bez.data.fill_mode = "NONE"
@@ -186,23 +223,38 @@ class BlenderPen(DrawablePenMixin, BasePen):
         return self
     
     def metallic(self, amount=1):
+        if not self.material == "auto" or not self.bsdf():
+            return
         self.bsdf().inputs[4].default_value = amount
         return self
     
+    def roughness(self, amount=0.5):
+        if not self.material == "auto" or not self.bsdf():
+            return
+        self.bsdf().inputs[7].default_value = amount
+        return self
+    
     def transmission(self, amount=1):
+        if not self.material == "auto" or not self.bsdf():
+            return
         self.bsdf().inputs[15].default_value = amount
         return self
     
     def subsurface(self, amount=0.01):
+        if not self.material == "auto" or not self.bsdf():
+            return
         self.bsdf().inputs[1].default_value = amount
         return self
 
-    def emission(self, color=None):
+    def emission(self, color=None, strength=1):
+        if not self.material == "auto" or not self.bsdf():
+            return
         if color is not None:
             self.setColorValue(self.bsdf().inputs[17].default_value, normalize_color(color))
+            self.bsdf().inputs[18].default_value = strength
         return self
     
-    def image(self, path):
+    def image(self, src=None, opacity=1, rect=None, pattern=True):
         mat = self.materials()[0]
         bsdf = self.bsdf()
         if "Image Texture" in mat.node_tree.nodes:
@@ -210,8 +262,12 @@ class BlenderPen(DrawablePenMixin, BasePen):
         else:
             imgtex = mat.node_tree.nodes.new("ShaderNodeTexImage")
             mat.node_tree.links.new(bsdf.inputs["Base Color"], imgtex.outputs["Color"])
-        imgtex.image = bpy.data.images.load(path)
+        imgtex.image = bpy.data.images.load(str(src))
         return self
+
+    def hide(self, hide):
+        self.bez.hide_viewport = hide
+        self.bez.hide_render = hide
     
     def convertToMesh(self):
         self.bez.select_set(True)
@@ -248,6 +304,24 @@ class BlenderPen(DrawablePenMixin, BasePen):
             self.bez.rotation_euler[2] = math.radians(z)
         return self
     
+    def set_origin(self, x, y, z):
+        #saved_location = bpy.context.scene.cursor.location
+        bpy.context.scene.cursor.location = Vector((x, y, z))
+        self.bez.select_set(True)
+        bpy.ops.object.origin_set(type='ORIGIN_CURSOR')
+        self.bez.select_set(False)
+        bpy.context.scene.cursor.location = Vector((0, 0, 0))
+        return self
+    
+    def with_origin(self, xyz, fn):
+        if xyz == "C":
+            pc = self.dat.ambit().pc
+            xyz = (pc.x/100, pc.y/100, 0)
+        self.set_origin(*xyz)
+        fn(self)
+        self.set_origin(0, 0, 0)
+        return self
+    
     def locate(self, x=None, y=None, z=None):
         if x is not None:
             self.bez.location[0] = x
@@ -257,14 +331,26 @@ class BlenderPen(DrawablePenMixin, BasePen):
             self.bez.location[2] = z
         return self
     
-    def draw(self, collection, style=None, scale=0.01, cyclic=True, dn=False, plane=False):
+    def draw(self, collection, style=None, scale=0.01, cyclic=True, dn=False, plane=False, material="auto"):
+        self.material = material
+
         if plane:
-            self.bez, self.created = BPH.Primitive("Plane", collection, self.tag, dn=dn, container=self.dat.getFrame().scale(scale))
+            self.bez, self.created = BPH.Primitive("Plane", collection, self.tag, dn=dn, material=material, container=self.dat.ambit().scale(scale))
         else:
-            self.bez, self.created = BPH.Primitive("Bezier", collection, self.tag, dn=dn)
+            self.bez, self.created = BPH.Primitive("Bezier", collection, self.tag, dn=dn, material=material)
+
+            if material and material != "auto":
+                mat = bpy.data.materials[material]
+                #print(">MAT", mat)
+                self.bez.data.materials.clear()
+                self.bez.data.materials.append(mat)
+
             self.bez.data.fill_mode = "BOTH"
             self.record(self.dat.copy().removeOverlap().scale(scale, point=False))
-            self.drawOnBezierCurve(self.bez.data, cyclic=cyclic)
+            try:
+                self.drawOnBezierCurve(self.bez.data, cyclic=cyclic)
+            except:
+                pass
         for attrs, attr in self.findStyledAttrs(style):
             self.applyDATAttribute(attrs, attr)
         return self
@@ -285,3 +371,6 @@ class BlenderPen(DrawablePenMixin, BasePen):
                 pt.co = BPH.Vector(c)
                 pt.handle_left = BPH.Vector(l)
                 pt.handle_right = BPH.Vector(r)
+
+    def noop(self, *args, **kwargs):
+        return self
\ No newline at end of file
