@@ -1,19 +1,16 @@
 from fontTools.pens.transformPen import TransformPen
-from fontTools.misc.transform import Transform
 from fontTools.pens.basePen import BasePen
 from pathlib import Path
 
 try:
     import cairo
 except:
-    pass
+    print(">>> No cairo installation found!")
 
 
-from drafting.geometry import Rect, Edge, Point
 from coldtype.pens.drawablepen import DrawablePenMixin
-from drafting.color import Color, Gradient
+from coldtype.color import Color, Gradient
 from coldtype.beziers import raise_quadratic
-import base64
 
 
 class CairoPen(DrawablePenMixin, BasePen):
@@ -108,7 +105,7 @@ class CairoPen(DrawablePenMixin, BasePen):
                 print("Should write to base64 and return — not yet supported")
         elif ip.suffix == ".pdf":
             surface = cairo.PDFSurface(str(ip), int(rect.w), int(rect.h))
-            print(surface)
+            #print(surface)
             ctx = cairo.Context(surface)
             #ctx.scale(2, 2)
             for pen in CairoPen.FindPens(pens):
