@@ -2,10 +2,12 @@ try:
     import drawBot as db
 except:
     pass
-from coldtype.pens.datpen import DATPen
-from drafting.geometry import Rect, Edge, Point
-from coldtype.pens.drawablepen import DrawablePenMixin, Gradient
-from drafting.color import Color
+
+from coldtype.pens.draftingpen import DraftingPen
+from coldtype.pens.draftingpens import DraftingPens
+from coldtype.geometry import Rect, Edge, Point
+from coldtype.pens.drawablepen import DrawablePenMixin
+from coldtype.color import Color, Gradient
 
 
 def get_image_rect(src):
@@ -13,7 +15,7 @@ def get_image_rect(src):
     return Rect(0, 0, w, h)
 
 
-class DrawBotPen(DrawablePenMixin):
+class DrawBotPen(DrawablePenMixin, DraftingPen):
     def __init__(self, dat, rect=None):
         super().__init__()
         self.rect = rect
@@ -85,7 +87,7 @@ class DrawBotPen(DrawablePenMixin):
     
     def shadow(self, clip=None, radius=10, alpha=0.3, color=Color.from_rgb(0,0,0,1)):
         if clip:
-            cp = DATPen(fill=None).rect(clip)
+            cp = DraftingPen(clip).f(None)
             bp = db.BezierPath()
             cp.replay(bp)
             db.clipPath(bp)
@@ -100,56 +102,25 @@ class DrawBotPen(DrawablePenMixin):
         stops = gradient.stops
         db.linearGradient(stops[0][1], stops[1][1], [list(s[0]) for s in stops], [0, 1])
     
-    def draw(self, scale=2, style=None):
-        with db.savedState():
-            db.scale(scale)
-            for attrs, attr in self.findStyledAttrs(style):
-                self.applyDATAttribute(attrs, attr)
-            db.drawPath(self.bp)
+    def draw(self, scale=1, style=None):
+        if hasattr(self.dat, "_pens"):
+            for p in self.dat._pens:
+                DrawBotPen(p, rect=self.rect).draw(scale=scale)
+        else:
+            with db.savedState():
+                db.scale(scale)
+                for attrs, attr in self.findStyledAttrs(style):
+                    self.applyDATAttribute(attrs, attr)
+                db.drawPath(self.bp)
+        return self
     
-    def Composite1(pens, rect, save_to, paginate=False, scale=2):
-        db.newDrawing()
-        rect = rect.scale(scale)
-        if not paginate:
-            db.newPage(rect.w, rect.h)
-        for pen in DrawBotPen.FindPens(pens):
-            if paginate:
-                db.newPage(rect.w, rect.h)
-            DrawBotPen(pen, rect).draw(scale=scale)
-        db.saveImage(str(save_to))
-        db.endDrawing()
-    
-    def Composite(pens, rect, save_to, scale=2):
-        db.newDrawing()
-        rect = rect.scale(scale)
-        db.newPage(rect.w, rect.h)
-
-        def draw(pen, state, data):
-            if state == 0:
-                DrawBotPen(pen, rect).draw(scale=scale)
-            elif state == -1:
-                imgf = pen.data.get("imgf")
-                if imgf:
-                    im = db.ImageObject()
-                    im.lockFocus()
-                    db.size(rect.w+300, rect.h+300)
-                    db.translate(150, 150)
-                    db.scale(scale)
-                    pen.data["im"] = im
-            elif state == 1:
-                imgf = pen.data.get("imgf")
-                im = pen.data.get("im")
-                if imgf and im:
-                    im.unlockFocus()
-                    imgf(im)
-                    x, y = im.offset()
-                    db.translate(-150, -150)
-                    db.image(im, (x, y))
-        
-        if isinstance(pens, DATPen):
-            pens = [pens]
-        for dps in pens:
-            dps.walk(draw)
-        
-        db.saveImage(str(save_to))
-        db.endDrawing()
+    def draw_with_filters(self, rect, filters):
+        im = db.ImageObject()
+        with im:
+            db.size(*rect.wh())
+            self.draw()
+        for filter_name, filter_kwargs in filters:
+            getattr(im, filter_name)(**filter_kwargs)
+        x, y = im.offset()
+        db.image(im, (x, y))
+        return self
\ No newline at end of file
