@@ -1,14 +1,23 @@
 from enum import Enum
 from fontTools.pens.filterPen import ContourFilterPen
 from fontTools.pens.recordingPen import RecordingPen
+from fontTools.misc.bezierTools import splitCubicAtT, calcCubicArcLength
+
 
 USE_SKIA_PATHOPS = True
 
-from pathops import Path, OpBuilder, PathOp
+try:
+    from pathops import Path, OpBuilder, PathOp
+except ImportError:
+    USE_SKIA_PATHOPS = False
+
 try:
     from booleanOperations.booleanGlyph import BooleanGlyph
-except:
-    pass
+    from booleanOperations.exceptions import UnsupportedContourError
+except ImportError:
+    if not USE_SKIA_PATHOPS:
+        print(">>> NO PATHOPS FOUND; please install either skia-pathops or booleanOperations")
+
 
 class BooleanOp(Enum):
     Difference = 0
@@ -59,11 +68,22 @@ def calculate_pathop(pen1, pen2, operation):
         result.draw(d0)
         return d0.value
     else:
+        bg = BooleanGlyph()
+        pen1.replay(bg.getPen())
+        if operation == BooleanOp.Simplify:
+            # ignore pen2
+            try:
+                bg = bg.removeOverlap()
+            except UnsupportedContourError:
+                print("booleanOperations could not removeOverlap (qcurve present)")
+                pass
+            dp = RecordingPen()
+            bg.draw(dp)
+            return dp.value
+
         bg2 = BooleanGlyph()
         if pen2:
             pen2.replay(bg2.getPen())
-        bg = BooleanGlyph()
-        pen1.replay(bg.getPen())
         bg = bg._booleanMath(BooleanOp.BooleanGlyphMethod(operation), bg2)
         dp = RecordingPen()
         bg.draw(dp)
@@ -72,11 +92,11 @@ def calculate_pathop(pen1, pen2, operation):
 
 class ExplodingPen(ContourFilterPen):
     def __init__(self, outPen):
-        self.pens = []
+        self._pens = []
         super().__init__(outPen)
 
     def filterContour(self, contour):
-        self.pens.append(contour)
+        self._pens.append(contour)
         return contour
 
 
