@@ -3,15 +3,14 @@ import skia, struct, tempfile
 from pathlib import Path
 
 from fontTools.pens.transformPen import TransformPen
-from fontTools.misc.transform import Transform
 from fontTools.pens.basePen import BasePen
 
-from coldtype.pens.datpen import DATPen, DATPens
+from coldtype.pens.datpen import DATPen
 from coldtype.pens.dattext import DATText
-from coldtype.pens.datimage import DATImage
-from drafting.geometry import Rect, Edge, Point
+from coldtype.img.datimage import DATImage
+from coldtype.geometry import Rect, Point
 from coldtype.pens.drawablepen import DrawablePenMixin, Gradient
-from drafting.color import Color
+from coldtype.color import Color
 from coldtype.text.reader import Style
 
 
@@ -59,7 +58,7 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
                 if "AntiAlias" not in skia_paint_kwargs:
                     skia_paint_kwargs["AntiAlias"] = True
             elif method == "blendmode":
-                self.blendmode = args[0]
+                self.blendmode = args[0].to_skia()
 
         for attrs, attr in all_attrs:
             filtered_paint_kwargs = {}
@@ -161,7 +160,10 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
             else:
                 # TODO scale the image, or maybe that shouldn't be here? this scaling method is horrible for image quality
                 self.canvas.scale(sx, sy)
+            was_alpha = self.paint.getAlphaf()
+            self.paint.setAlphaf(was_alpha*self.alpha)
             self.canvas.drawImage(image, bx/sx, by/sy, self.paint)
+            self.paint.setAlphaf(was_alpha)
             self.canvas.restore()
             return True
     
@@ -177,15 +179,16 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
                 self.canvas.clipPath(sp.path, doAntiAlias=True)
         self.paint.setColor(skia.ColorBLACK)
         self.paint.setImageFilter(skia.ImageFilters.DropShadow(0, 0, radius, radius, color.skia()))
-        #self.paint.setBlendMode(skia.BlendMode.kClear)
         return
     
     def Composite(pens, rect, save_to, scale=1, context=None, style=None):
+        rect = rect.scale(scale)
+
         if context:
             info = skia.ImageInfo.MakeN32Premul(rect.w, rect.h)
             surface = skia.Surface.MakeRenderTarget(context, skia.Budgeted.kNo, info)
         else:
-            print("CPU RENDER")
+            #print("CPU RENDER")
             surface = skia.Surface(rect.w, rect.h)
         
         with surface as canvas:
@@ -222,7 +225,6 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
             pens.scale(scale, scale, Point((0, 0)))
         
         if not pens.visible:
-            print("HERE!")
             return
         
         def draw(pen, state, data):
@@ -266,17 +268,15 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
                         deg, pt = args
                         canvas.rotate(-deg, pt.x, pt.y)
                     #print(action, args)
+                paint.setAlphaf(paint.getAlphaf()*data["alpha"])
                 canvas.drawImage(pen._img, f.x, f.y, paint)
                 canvas.restore()
                 #pen = DATPen().rect(pen.bounds()).img(pen._img, rect=pen.bounds(), pattern=False)
                 return
             
             if state == 0:
-                #print("DRAWING", pen)
-                SkiaPen(pen, rect, canvas, scale, style=style, alpha=pen.calc_alpha())
+                SkiaPen(pen, rect, canvas, scale, style=style, alpha=data["alpha"])
         
-        #print("COMPTOCANV >>>>>>>>>>>>>>>>")
-        #pens.print_tree()
         pens.walk(draw, visible_only=True)
     
     def Precompose(pens, rect, fmt=None, context=None, scale=1, disk=False):
@@ -286,7 +286,7 @@ class SkiaPen(DrawablePenMixin, SkiaPathPen):
             surface = skia.Surface.MakeRenderTarget(context, skia.Budgeted.kNo, info)
             assert surface is not None
         else:
-            print("CPU PRECOMPOSE")
+            #print("CPU PRECOMPOSE")
             surface = skia.Surface(rect.w, rect.h)
         
         with surface as canvas:
