@@ -1,12 +1,13 @@
-import sys, os
-import ast, threading
-
+import ast, json, threading, platform, os
+from http.server import SimpleHTTPRequestHandler
 from enum import Enum
-from pathlib import Path
 
-from SimpleWebSocketServer import SimpleWebSocketServer, WebSocket
+try:
+    from SimpleWebSocketServer import SimpleWebSocketServer, WebSocket
+except ImportError:
+    SimpleWebSocketServer = object
+    WebSocket = object
 
-WEBSOCKET_PORT = 8007
 
 class Watchable(Enum):
     Source = "Source"
@@ -37,20 +38,38 @@ class bcolors:
 
 class SimpleEcho(WebSocket):
     def handleMessage(self):
-        #print("INCOMING", self, self.data)
+        if "webviewer" in self.data:
+            data = json.loads(self.data)
+            if data.get("webviewer") == True:
+                self.webviewer = True
+        #print("INCOMING!", self, self.address, self.data)
         self.messages.append(self.data)
 
     def handleConnected(self):
-        print(self.address, 'connected')
+        #if not str(self.address).startswith("('::ffff"):
+        #    print(self.address, "connected")
         self.messages = []
+        self.webviewer = False
 
     def handleClose(self):
-        print(self.address, 'closed')
+        #print(self.address, "closed")
+        pass
 
 
-def echo_server():
-    return SimpleWebSocketServer('', WEBSOCKET_PORT, SimpleEcho)
+def echo_server(port):
+    return SimpleWebSocketServer('', port, SimpleEcho)
 
+def run_echo_server(port, name):
+    try:
+        server = echo_server(port)
+        daemon = threading.Thread(name=name,
+            target=server.serveforever)
+        daemon.setDaemon(True)
+        daemon.start()
+        print(f"<coldtype: websocket-opened@localhost:{port}>")
+        return server
+    except OSError:
+        return None
 
 def bytesto(bytes):
     r = float(bytes)
@@ -79,4 +98,8 @@ def file_and_line_to_def(filepath, lineno):
             else:
                 candidate = item
     if candidate:
-        return candidate.name
\ No newline at end of file
+        return candidate.name
+    
+def play_sound(name="Pop"):
+    if platform.system() == "Darwin":
+        os.system(f"afplay /System/Library/Sounds/{name}.aiff")
\ No newline at end of file
