@@ -1,9 +1,12 @@
-from coldtype.pens.datpen import DATPen, DATPens
-from drafting.geometry import Rect, Point
+#from coldtype.pens.draftingpens import DraftingPen, DraftingPens
+from coldtype.pens.draftingpens import DraftingPens
+from coldtype.geometry import Rect, Point
 
 from coldtype.text.shaper import segment
-from coldtype.text.reader import Style, StyledString, FittableMixin, Font, normalize_font_path, SegmentedString
+from coldtype.text.reader import Style, StyledString, FittableMixin, Font, normalize_font_path, SegmentedString,  _PenClass, _PensClass
 
+import inspect
+from collections import namedtuple
 
 class GrafStyle():
     def __init__(self, leading=10, x="centerx", xp=0, width=0, **kwargs):
@@ -16,11 +19,13 @@ class GrafStyle():
 class Graf():
     def __init__(self, lines, container, style=None, **kwargs):
         if isinstance(container, Rect):
-            self.container = DATPen().rect(container)
+            self.container = _PenClass().rect(container)
         else:
             self.container = container
         if style and isinstance(style, GrafStyle):
             self.style = style
+        elif style and isinstance(style, int):
+            self.style = GrafStyle(leading=style)
         else:
             self.style = GrafStyle(**kwargs)
         self.lines = lines
@@ -29,7 +34,7 @@ class Graf():
         # which came first, the height or the width???
         rects = []
         leadings = []
-        box = self.container.getFrame()
+        box = self.container.ambit()
         leftover = box
         for l in self.lines:
             box, leftover = leftover.divide(l.height(), "maxy", forcePixel=True)
@@ -60,7 +65,7 @@ class Graf():
     
     def pens(self, align=False):
         rects = self.lineRects()
-        pens = DATPens()
+        pens = _PensClass()
         for idx, l in enumerate(self.lines):
             r = rects[idx]
             dps = l.pens().translate(r.x, r.y) # r.x
@@ -72,7 +77,7 @@ class Graf():
         return pens
     
     def fpa(self, rect=None):
-        return self.fit().pens().align(rect or self.container.getFrame())
+        return self.fit().pens().align(rect or self.container.ambit())
 
 
 class Lockup(FittableMixin):
@@ -110,18 +115,18 @@ class Lockup(FittableMixin):
                 if self.nestSlugs:
                     pens.append(dps)
                 else:
-                    pens.extend(dps.pens)
+                    pens.extend(dps._pens)
             else:
                 dps = s.pen()
                 dps.translate(x_off, 0)
                 pens.append(dps)
-            x_off += dps.getFrame().w
+            x_off += dps.ambit(th=0).w
             try:
                 x_off += s.margin[1]
                 x_off += s.strings[-1].tracking
             except:
                 pass
-        return DATPens(pens)
+        return _PensClass(pens)
     
     def pen(self):
         return self.pens().pen()
@@ -176,7 +181,7 @@ class Composer():
         if fit is not None:
             self.graf.fit(fit)
     
-    def pens(self) -> DATPens:
+    def pens(self):
         """
         Structured representation of the multi-line text
         
@@ -184,8 +189,124 @@ class Composer():
         """
         return self.graf.pens()
     
-    def pen(self) -> DATPen:
+    def pen(self):
         """
         Entire multiline text as a single vector
         """
-        return self.graf.pens().pen()
\ No newline at end of file
+        return self.graf.pens().pen()
+
+
+def StSt(text,
+    font,
+    font_size=24,
+    rect=Rect(1080, 1080),
+    xa="mdx",
+    **kwargs):
+    
+    if isinstance(font, Style):
+        style = font
+    else:
+        style = Style(font, font_size, **kwargs)
+    
+    fit = kwargs.get("fit", None)
+    leading = kwargs.get("leading", 10)
+
+    if "\n" in text:
+        lockup = Composer(rect, text, style, fit=fit, leading=leading).pens()
+        if xa:
+            lockup = lockup.xa(xa)
+        for l in lockup:
+            l._frame = None
+    else:
+        lockup = StyledString(text, style)
+        if fit:
+            lockup.fit(fit)
+        lockup = lockup.pens()
+    return lockup
+
+GlyphwiseGlyph = namedtuple("GlyphwiseGlyph", ["i", "c", "e"])
+
+def Glyphwise(st, styler):
+    # TODO possible to have an implementation
+    # aware of a non-1-to-1 mapping of characters
+    # to glyphs? seems very difficult if not impossible,
+    # since it requires mapping the string to glyphs
+    # and then somehow mapping the glyphs back to the
+    # equivalent string (?) in order to get the proper
+    # kerning information for the sub-strings —
+    # it maybe possible to pass glyph-id's directly
+    # to harfbuzz, which would solve the problem,
+    # though that does seem kind of hard to believe
+
+    #glyphs = StyledString(st, styler(0, st[0])).glyphs
+    #print(glyphs)
+    #print([g.name for g in glyphs])
+
+    def kx(dps, idx):
+        return dps[idx].ambit().x
+    
+    def krn(off, on, idx):
+        return kx(off, idx) - kx(on, idx)
+
+    arg_count = len(inspect.signature(styler).parameters)
+    dps = DraftingPens()
+    prev = 0
+    tracks = []
+
+    for idx, c in enumerate(st):
+        #c = gi.name
+        test = c
+        target = 0
+        tcount = 1
+        if idx < len(st) - 1:
+            test = [test, st[idx+1]]
+            tcount += 1
+        if idx > 0:
+            test = [st[idx-1], test]
+            target = 1
+            tcount += 1
+        if isinstance(test, str):
+            test = [test]
+        
+        e = idx / (len(st)-1)
+        gg = GlyphwiseGlyph(idx, c, e)
+
+        if arg_count == 1:
+            skon = styler(gg)
+        else:
+            skon = styler(idx, gg)
+        
+        skoff = skon.mod(kern=0)
+
+        #test_list = [t for sublist in test for t in sublist]
+        #print(test_list)
+        test_str = "".join([t if isinstance(t, str) else "".join(t) for t in test])
+        #print(c, test_str, target, test, tcount)
+
+        tkon = StSt(test_str, skon)
+        tkoff = StSt(test_str, skoff)
+
+        if idx == 0:
+            dps.append(tkoff[0])
+            prev = krn(tkoff, tkon, 1)
+        if target > 0:
+            _prev = krn(tkoff, tkon, 1)
+            prev_av = (prev+_prev)/2
+            #print(prev_av, kx(tkoff, 1))
+
+            dps.append(tkoff[1].copy(with_data=True)
+                .translate(-kx(tkoff, 1), 0))
+            tracks.append(-prev_av)
+
+            if tcount > 2:
+                #print("_PREV", _prev)
+                _next = krn(tkoff, tkon, 2) - _prev #tkoff[0].ambit().w
+                #print("next", _next)
+            else:
+                _next = 0
+            prev = _next
+    
+    #print(tracks)
+    return dps.distribute(
+        tracks=tracks
+    )
\ No newline at end of file
