@@ -1,8 +1,8 @@
 from pathlib import Path
 from collections import OrderedDict
-from functools import partial
+from functools import partial, lru_cache
 
-import unicodedata, os, math, asyncio
+import unicodedata, math, os, re
 
 from fontTools.misc.transform import Transform
 from fontTools.pens.transformPen import TransformPen
@@ -11,22 +11,28 @@ from fontTools.pens.recordingPen import RecordingPen, replayRecording
 from fontTools.pens.boundsPen import ControlBoundsPen, BoundsPen
 from fontTools.ttLib.ttFont import TTFont
 
-from drafting.color import normalize_color
+from coldtype.color import normalize_color
+from coldtype.pens.draftingpen import DraftingPen
+from coldtype.pens.draftingpens import DraftingPens
 from coldtype.pens.datpen import DATPen, DATPens
-from drafting.geometry import Rect, Point
+from coldtype.geometry import Rect, Point
 
 from typing import Optional, Callable, Union
 
-try:
-    from fontgoggles.font import getOpener
-    from fontgoggles.font.baseFont import BaseFont
-    from fontgoggles.font.otfFont import OTFFont
-    from fontgoggles.misc.textInfo import TextInfo
-except ModuleNotFoundError:
-    from coldtype.fontgoggles.font import getOpener
-    from coldtype.fontgoggles.font.baseFont import BaseFont
-    from coldtype.fontgoggles.font.otfFont import OTFFont
-    from coldtype.fontgoggles.misc.textInfo import TextInfo
+_PenClass = DATPen
+_PensClass = DATPens
+
+# try:
+#     from fontgoggles.font import getOpener
+#     from fontgoggles.font.baseFont import BaseFont
+#     from fontgoggles.font.otfFont import OTFFont
+#     from fontgoggles.misc.textInfo import TextInfo
+# except ModuleNotFoundError:
+
+from coldtype.fontgoggles.font import getOpener
+from coldtype.fontgoggles.font.baseFont import BaseFont
+from coldtype.fontgoggles.font.otfFont import OTFFont
+from coldtype.fontgoggles.misc.textInfo import TextInfo
 
 
 class FittableMixin():
@@ -59,6 +65,13 @@ _prefixes = [
     ["", "/Library/Fonts"]
 ]
 
+# TODO windows & linux?
+ALL_FONT_DIRS = [
+    "/System/Library/Fonts",
+    "/Library/Fonts",
+    "~/Library/Fonts",
+]
+
 class FontNotFoundException(Exception):
     pass
 
@@ -67,18 +80,17 @@ def normalize_font_prefix(path_string):
         path_string = path_string.replace(prefix, expansion)
     return Path(path_string).expanduser().resolve()
 
-def normalize_font_path(font):
+def normalize_font_path(font, nonexist_ok=False):
     global _prefixes
     literal = normalize_font_prefix(str(font))
     ufo = literal.suffix == ".ufo"
+    if nonexist_ok:
+        return str(literal)
     if literal.exists() and (not literal.is_dir() or ufo):
         return str(literal)
     else:
         raise FontNotFoundException(literal)
 
-def empty_writer(*args):
-    pass
-
 FontCache = {}
 
 class Font():
@@ -96,15 +108,56 @@ class Font():
         if self._loaded:
             return self
         else:
+            self.font.load(None)
             self._loaded = True
-            asyncio.get_event_loop().run_until_complete(self.font.load(empty_writer))
             return self
     
+    @staticmethod
     def Cacheable(path):
         if path not in FontCache:
             FontCache[path] = Font(path, cacheable=True).load()
         return FontCache[path]
 
+    @lru_cache()
+    def List(regex, regex_dir=None):
+        results = []
+        for dir in ALL_FONT_DIRS:
+            dir = normalize_font_prefix(dir)
+            if regex_dir:
+                if not re.search(regex_dir, str(dir)):
+                    continue
+            for root, dirs, files in os.walk(dir):
+                for dir in dirs:
+                    path = Path(root + "/" + dir)
+                    if path.suffix == ".ufo":
+                        if re.search(regex, dir):
+                            results.append(path)
+                for file in files:
+                    if re.search(regex, file):
+                        path = Path(root + "/" + file)
+                        if path.suffix in [".ttf", ".otf", ".ttc"]:
+                            results.append(path)
+        return results
+
+    def Find(regex, regex_dir=None, index=0):
+        found = Font.List(regex, regex_dir)
+        try:
+            return Font.Cacheable(found[index])
+        except:
+            raise FontNotFoundException()
+    
+    @staticmethod
+    def ColdtypeObviously():
+        return Font.Cacheable(Path(__file__).parent.parent / "demo/ColdtypeObviously-VF.ttf")
+
+    @staticmethod
+    def MutatorSans():
+        return Font.Cacheable(Path(__file__).parent.parent / "demo/MutatorSans.ttf")
+    
+    @staticmethod
+    def RecursiveMono():
+        return Font.Cacheable(Path(__file__).parent.parent / "demo/RecMono-CasualItalic.ttf")
+
 class Style():
     """
     Class for configuring font properties
@@ -146,6 +199,7 @@ class Style():
             rotate=0,
             reverse=False,
             removeOverlap=False,
+            q2c=False,
             lang=None,
             narrower=None,
             include_blanks=False,
@@ -154,6 +208,7 @@ class Style():
             data={},
             layer=None,
             liga=True,
+            kern=True,
             fill=(0, 0.5, 1),
             stroke=None,
             strokeWidth=0,
@@ -167,7 +222,7 @@ class Style():
             varyFontSize=False,
             preventHwid=False,
             fitHeight=None,
-            load_font=True, # should Coldtype attempt to load the font?
+            load_font=True, # should we attempt to load the font?
             tag=None, # way to differentiate in __eq__
             **kwargs):
 
@@ -189,6 +244,7 @@ class Style():
         self.layer = layer
         self.reverse = kwargs.get("r", reverse)
         self.removeOverlap = kwargs.get("ro", removeOverlap)
+        self.q2c = q2c
         self.rotate = rotate
         self.include_blanks = include_blanks
         self.scaleVariations = scaleVariations
@@ -246,7 +302,7 @@ class Style():
                 if k == 0:
                     found_features[k] = 0
         
-        self.features = {**dict(kern=True, liga=liga), **found_features}
+        self.features = {**dict(kern=kern, liga=liga), **found_features}
 
         self.fill = normalize_color(fill)
         self.stroke = normalize_color(stroke)
@@ -333,8 +389,9 @@ class Style():
                 variations[k] = axis.maxValue
             elif v == "default":
                 variations[k] = axis.defaultValue
-            elif scale and v <= 1.0:
-                variations[k] = float(abs(axis.maxValue-axis.minValue)*v + axis.minValue)
+            elif scale:
+                _v = max(0, min(1, v))
+                variations[k] = float(abs(axis.maxValue-axis.minValue)*_v + axis.minValue)
             else:
                 if v < axis.minValue or v > axis.maxValue:
                     variations[k] = max(axis.minValue, min(axis.maxValue, v))
@@ -353,7 +410,7 @@ class Style():
         def stretcher(w, xp, i, p):
             np = (p.flatten(flatten) if flatten else p).nonlinear_transform(lambda x,y: (x if x < xp else x + w, y))
             if debug:
-                (np.record(DATPen()
+                (np.record(_PenClass()
                     .line([(xp, -250), (xp, 1000)])
                     .outline()))
             return np
@@ -372,10 +429,10 @@ class Style():
             np = (p.flatten(flatten) if flatten else p).nonlinear_transform(lambda x,y: (x if is_left((xdsc, -250), (xasc, 1000), (x, y)) else x + w, y))
             if debug:
                 (np
-                    .record(DATPen()
+                    .record(_PenClass()
                         .line([(xdsc, -250), (xasc, 1000)])
                         .outline())
-                    .record(DATPen()
+                    .record(_PenClass()
                         .moveTo((x0+50/2, y0+50/2))
                         .dots(radius=50)))
             return np
@@ -399,8 +456,8 @@ class Style():
             elif align == "mxy":
                 np.translate(0, -h)
             if debug:
-                (np.record(DATPen()
-                    .line([(0, yp), (p.getFrame().point("E").x, yp)])
+                (np.record(_PenClass()
+                    .line([(0, yp), (p.ambit().point("E").x, yp)])
                     .outline()))
             return np
 
@@ -413,17 +470,17 @@ class Style():
             x0, y0 = xy
             ra = math.radians(90+angle)
             ydsc = y0 + (0 - x0) / math.tan(ra)
-            yasc = y0 + (p.getFrame().point("E").x - x0) / math.tan(ra)
+            yasc = y0 + (p.ambit().point("E").x - x0) / math.tan(ra)
             p0 = (0, ydsc)
-            p1 = (p.getFrame().point("E").x, yasc)
+            p1 = (p.ambit().point("E").x, yasc)
 
             np = (p.flatten(flatten) if flatten else p).nonlinear_transform(lambda x,y: (x, y if not is_left(p0, p1, (x, y)) else y + h))
             if debug:
                 (np
-                    .record(DATPen()
+                    .record(_PenClass()
                         .line([p0, p1])
                         .outline())
-                    .record(DATPen()
+                    .record(_PenClass()
                         .moveTo((x0+50/2, y0+50/2))
                         .dots(radius=50)))
             return np
@@ -525,6 +582,7 @@ class StyledString(FittableMixin):
             self.trackFrames(space_width=space_width)
 
         for glyph in self.glyphs:
+            #print(glyph, glyph.frame)
             glyph.frame = glyph.frame.scale(self.scale())
 
         if self.style.trackingMode == 0:
@@ -670,9 +728,9 @@ class StyledString(FittableMixin):
         t = t.scale(s)
         t = t.translate(glyph.frame.x/self.scale(), glyph.frame.y/self.scale())
         #t = t.translate(0, bs)
-        out_pen = DATPen()
+        out_pen = _PenClass()
         tp = TransformPen(out_pen, (t[0], t[1], t[2], t[3], t[4], t[5]))
-        ip = DATPen().record(in_pen)
+        ip = _PenClass().record(in_pen)
         if self.style.mods and glyph.name in self.style.mods:
             w, mod = self.style.mods[glyph.name]
             mod(-1, ip)
@@ -691,20 +749,22 @@ class StyledString(FittableMixin):
         return out_pen
     
     def _emptyPenWithAttrs(self):
-        attrs = dict(fill=self.style.fill)
+        #attrs = dict(fill=self.style.fill)
+        #if self.style.stroke:
+        #    attrs["stroke"] = dict(color=self.style.stroke, weight=self.style.strokeWidth)
+        dp = _PenClass().f(self.style.fill)
         if self.style.stroke:
-            attrs["stroke"] = dict(color=self.style.stroke, weight=self.style.strokeWidth)
-        dp = DATPen(**attrs)
+            dp.s(self.style.stroke).sw(self.style.strokeWidth)
         return dp
 
-    def pens(self, frame=True) -> DATPens:
+    def pens(self, frame=True) -> DraftingPens:
         """
         Vectorize text into a ``DATPens``, such that each glyph (or ligature) is represented by a single `DATPen` (or a ``DATPens`` in the case of a color font, which will then nest a `DATPen` for each layer of that color glyph)
         """
         self.resetGlyphRun()
         self.style.font.font.addGlyphDrawings(self.glyphs, colorLayers=True)
         
-        pens = DATPens()
+        pens = _PensClass()
         for idx, g in enumerate(self.glyphs):
             dp_atom = self._emptyPenWithAttrs()
             if len(g.glyphDrawing.layers) == 1:
@@ -712,19 +772,29 @@ class StyledString(FittableMixin):
                 dp_atom.typographic = True
                 dp_atom.addFrame(g.frame)
                 dp_atom.glyphName = g.name
+                if self.style.q2c:
+                    dp_atom.q2c()
                 if self.style.removeOverlap:
                     dp_atom.removeOverlap()
             else:
-                dp_atom = DATPens()
+                dp_atom = _PensClass()
                 dp_atom.layered = True
-                for layer in g.glyphDrawing.layers:
+                for lidx, layer in enumerate(g.glyphDrawing.layers):
                     dp_layer = self._emptyPenWithAttrs()
                     #dp_layer.value = layer[0].value
                     dp_layer.value = self.scalePenToStyle(g, layer[0]).value
-                    dp_layer.f(self.style.font.font.colorPalettes[self.style.palette][layer[1]])
-                    dp_atom += dp_layer
+                    if isinstance(self.style.palette, int):
+                        dp_layer.f(self.style.font.font.colorPalettes[self.style.palette][layer[1]])
+                    else:
+                        dp_layer.f(self.style.palette[layer[1]])
+                    if len(dp_layer.value) > 0:
+                        #dp_layer.addFrame(g.frame, typographic=True)
+                        dp_layer.glyphName = f"{g.name}_layer_{lidx}"
+                        dp_atom += dp_layer
                 dp_atom.addFrame(g.frame, typographic=True)
                 dp_atom.glyphName = g.name
+            
+            #dp_atom._parent = pens
             pens.append(dp_atom, allow_blank=self.style.include_blanks)
 
         if self.style.reverse:
@@ -735,7 +805,7 @@ class StyledString(FittableMixin):
 
         return pens
 
-    def pen(self, frame=True) -> DATPen:
+    def pen(self, frame=True) -> DraftingPen:
         """
         Vectorize all text into single ``DATPen``
         """
@@ -773,14 +843,14 @@ class SegmentedString(FittableMixin):
         return adjusted
 
     def pens(self):
-        pens = DATPens()
+        pens = _PensClass()
         x_off = 0
         for s in self.strings:
             dps = s.pens(frame=True)
             if dps.layered:
                 pens.layered = True
             dps.translate(x_off, 0)
-            pens.extend(dps.pens)
-            x_off += dps.getFrame().w
+            pens.extend(dps._pens)
+            x_off += dps.ambit().w
         return pens
         #return DATPens([s.pens(frame=True) for s in self.strings])
\ No newline at end of file
