@@ -5,8 +5,8 @@ from coldtype.time.audio import Wavfile
 from coldtype.text import Style, StyledString
 from coldtype.pens.datpen import DATPen, DATPens
 from coldtype.pens.dattext import DATText
-from drafting.geometry import Rect
-from drafting.color import hsl, bw
+from coldtype.geometry import Rect
+from coldtype.color import hsl, bw
 from coldtype.renderable.animation import animation
 from coldtype.renderable import Overlay
 from contextlib import contextmanager
@@ -214,34 +214,6 @@ class Subtitler(Sequence):
     def process_state(self, rs, fi, clips=None):
         if not clips:
             clips = self.clips()
-
-        if rs.text:
-            self.add_clip(fi, rs.text, clips)
-        elif rs.cmd:
-            if rs.cmd == "d":
-                self.delete_clip(fi, clips)
-            elif rs.cmd == "c":
-                self.cut_clip(fi, clips)
-            elif rs.cmd == "p":
-                self.persist()
-            elif rs.cmd == "q":
-                #print(self.closest(fi, -1, clips))
-                self.closest_to_playhead(fi, -1, clips)
-            elif rs.cmd == "w":
-                #print(self.closest(fi, +1, clips))
-                self.closest_to_playhead(fi, +1, clips)
-            elif rs.cmd == "ts":
-                self.mod_clip_text(fi, clips,
-                    lambda s, t: "*" + t if s != "*" else t)
-            elif rs.cmd == "tl":
-                self.mod_clip_text(fi, clips,
-                    lambda s, t: "≈" + t if s != "≈" else t)
-            elif rs.cmd == "tc":
-                self.mod_clip_text(fi, clips,
-                    lambda s, t: "+" + t if s != "+" else t)
-            elif re.match(r"[0-9]{1}", rs.cmd):
-                self.data["workarea_track"] = int(rs.cmd)
-                self.persist()
     
     def clip_timeline(self, fi, far, sw=30, sh=50, font_name=None):
         seq = DATPens()
@@ -280,7 +252,6 @@ class lyric_editor(animation):
         super().__init__(
             rect=rect,
             timeline=timeline,
-            audio=timeline.audio_path,
             rstate=1,
             watch=[timeline.path],
             **kwargs)
