@@ -7,7 +7,9 @@ from coldtype.time.timeline import Timeline
 
 from coldtype.text import StyledString, Lockup, Graf, GrafStyle, Style
 from coldtype.pens.datpen import DATPen, DATPens
-from drafting.color import *
+from coldtype.color import *
+
+from coldtype.time.clip import Clip, ClipFlags, ClipType
 
 from typing import Optional, Union, Callable, Tuple
 
@@ -18,191 +20,13 @@ class Marker(Timeable):
         super().__init__(start, end, data=marker)
 
 
-class ClipType(Enum):
-    ClearScreen = "ClearScreen"
-    NewLine = "NewLine"
-    GrafBreak = "GrafBreak"
-    Blank = "Blank"
-    Isolated = "Isolated"
-    JoinPrev = "JoinPrev"
-    Meta = "Meta"
-
-
-class ClipFlags(Enum):
-    FadeIn = "FadeIn"
-    FadeOut = "FadeOut"
-
-
-class Clip(Timeable):
-    def __init__(self, text, start, end, idx=None, track=0):
-        self.idx = idx
-        self.input_text = str(text)
-        self.text = text
-        self.start = start
-        self.end = end
-
-        self.styles = []
-        self.style_clips = []
-        self.position = 1
-        self.joined = False
-        self.joinPrev = None
-        self.joinNext = None
-        self.track = track
-        self.group = None
-        self.blank = False
-        self.blank_height = 20
-        self.inline_styles = []
-        self.inline_data = {}
-        self.flags = {}
-        self.type = ClipType.Isolated
-
-        if self.text.startswith("*"):
-            self.text = self.text[1:]
-            self.type = ClipType.ClearScreen
-        elif self.text.startswith("≈"):
-            self.text = self.text[1:]
-            self.type = ClipType.NewLine
-        elif self.text.startswith("¶"):
-            self.text = self.text[1:]
-            self.type = ClipType.GrafBreak
-        elif self.text.startswith("+"):
-            self.text = self.text[1:]
-            self.type = ClipType.JoinPrev
-        elif self.text.startswith("µ:"):
-            self.type = ClipType.Meta
-            self.text = self.text[2:]
-        
-        parts = self.text.split(":")
-        inline_style_marker = parts.index("ß") if "ß" in parts else -1
-        inline_data_marker = parts.index("∂") if "∂" in parts else -1
-
-        if inline_style_marker > -1:
-            self.inline_styles = parts[inline_style_marker+1].split(",")
-        if inline_data_marker > -1:
-            qs = [q.split("=") for q in parts[inline_data_marker+1].split("&")]
-            for k, v in qs:
-                self.inline_data[k] = eval(v)
-        
-        self.text = parts[0]
-
-        #if "ß" in self.text:
-        #    parts = self.text.split(":")
-        #    self.text = parts[0]
-        #    self.inline_styles = parts[-1].split(",")
-        
-        #self.text = self.text.replace("§", "\u200b")
-        if self.text.startswith("§"):
-            #self.text = "\u200b"
-            self.text = self.text[1:]
-            #self.type = ClipType.JoinPrev
-            self.type = ClipType.NewLine
-            self.blank = True
-            try:
-                self.blank_height = float(self.text.strip())
-            except:
-                pass
-    
-        if self.text.startswith("∫"):
-            self.text = ""
-            self.blank = True
-        
-        if not self.text:
-            self.blank = True
-            self.text = ""
-        
-        if self.text.startswith("ƒ"):
-            r = r"^([0-9]+)ƒ"
-            self.text = self.text[1:]
-            value = 3
-            match = re.match(r, self.text)
-            if match:
-                value = int(match[1])
-                self.text = re.sub(r, "", self.text)
-            self.flags[ClipFlags.FadeIn] = value
-        
-        elif self.text.endswith("ƒ"):
-            self.flags[ClipFlags.FadeOut] = 3
-        
-        self.original_text = str(self.text)
-    
-    def addJoin(self, clip, direction):
-        if direction == -1:
-            self.joinPrev = clip
-        elif direction == 1:
-            self.joinNext = clip
-    
-    def joinStart(self):
-        if self.joinPrev:
-            return self.joinPrev.joinStart()
-        else:
-            return self.start
-    
-    def joinEnd(self):
-        if self.joinNext:
-            return self.joinNext.joinEnd()
-        else:
-            return self.end
-    
-    def textForIndex(self, index):
-        txt = self.text
-        try:
-            txt = self.text.split("/")[index]
-        except:
-            txt = self.text.split("/")[-1]
-        return txt, self.addSpace
-    
-    def style_matching(self, txt):
-        try:
-            return next(c for c in self.style_clips if txt in c.text)
-        except StopIteration:
-            return None
-
-    def ftext(self):
-        txt = self.text
-        if self.type == ClipType.Isolated:
-            return " " + txt
-        else:
-            return txt
-    
-    def fade(self, default_fade=5):
-        if ClipFlags.FadeIn in self.flags:
-            return self.flags[ClipFlags.FadeIn]
-        else:
-            return default_fade
-    
-    def fadeIn(self, fi, easefn="seio", fade_length=None, start=0):
-        if ClipFlags.FadeIn in self.flags or fade_length:
-            if fade_length:
-                fade = fade_length
-            else:
-                fade = self.flags[ClipFlags.FadeIn]
-            
-            if start == 0:
-                start_frame = self.start
-            elif start == -1:
-                start_frame = self.start - fade
-            
-            if fi < start_frame:
-                return -1
-            elif fi > start_frame + fade:
-                return 1
-            else:
-                fv = (fi - start_frame) / fade
-                a, _ = ease(easefn, fv)
-                return a
-        return -1
-    
-    def __repr__(self):
-        return "<Clip:({:s}/{:04d}/{:04d}\"{:s}\")>".format([" -1", "NOW", " +1"][self.position+1], self.start, self.end, self.text)
-
-
 class ClipGroupPens(DATPens):
     def __init__(self, clip_group):
         super().__init__()
         self.cg = clip_group
     
     def _iterate_tags(self, tag, pens):
-        if hasattr(pens, "pens"):
+        if hasattr(pens, "_pens"):
             for pen in pens:
                 if pen._tag == tag:
                     yield pen.data.get(tag), pen
@@ -222,19 +46,45 @@ class ClipGroupPens(DATPens):
         for clip, pen in self.iterate_clips():
             fn(clip, pen)
         return self
+    
+    def clean_empties(self):
+        for _, pen in self.iterate_slugs():
+            pen._pens = [p for p in pen._pens if len(p._pens) > 0]
+        for line in self:
+            line._pens = [p for p in line._pens if len(p._pens) > 0]
+        self._pens = [p for p in self._pens if len(p._pens) > 0]
+        return self
 
     def remove_futures(self, clean=True):
         for clip, pen in self.iterate_clips():
             if clip.position > 0:
-                pen.pens = []
+                pen._pens = []
         
         if clean:
-            for _, pen in self.iterate_slugs():
-                pen.pens = [p for p in pen.pens if len(p.pens) > 0]
-            for line in self:
-                line.pens = [p for p in line.pens if len(p.pens) > 0]
-            self.pens = [p for p in self.pens if len(p.pens) > 0]
+            self.clean_empties()
+        return self
+    
+    def remove_future_lines(self, clean=True):
+        for _, line in self.iterate_lines():
+            any_now = False
+            for pen in line:
+                for p in pen:
+                    clip = p.data.get("clip")
+                    if clip:
+                        if clip.position <= 0:
+                            any_now = True
+            
+            if not any_now:
+                line._pens = []
+            #for x in line.iterate_clips():
+            #    print(">>>", x)
+
+        #for clip, pen in self.iterate_clips():
+        #    if clip.position > 0:
+        #        pen._pens = []
         
+        if clean:
+            self.clean_empties()
         return self
     
     def understroke(self, s=0, sw=5):
@@ -247,22 +97,26 @@ class ClipGroup(Timeable):
     def __init__(self, timeline, index, clips):
         self.index = index
         self.clips = clips
-        if len(clips) > 0:
-            self.start = clips[0].start
-            self.end = clips[-1].end
-            self.track = clips[0].track
+        self.timeline = timeline
+        self.style_indices = []
+
+        self.retime()
+
+        for idx, clip in enumerate(clips):
+            clip.idx = idx
+            clip.group = self
+    
+    def retime(self):
+        if len(self.clips) > 0:
+            self.start = self.clips[0].start
+            self.end = self.clips[-1].end
+            self.track = self.clips[0].track
             self.valid = True
         else:
             self.start = 0
             self.end = 0
             self.track = None
             self.valid = False
-        self.timeline = timeline
-        self.style_indices = []
-
-        for idx, clip in enumerate(clips):
-            clip.idx = idx
-            clip.group = self
     
     def styles(self):
         all_styles = set()
@@ -515,7 +369,7 @@ class ClipGroup(Timeable):
                     if clip.type == ClipType.JoinPrev and last_clip_dps:
                         grouped_clip_dps = last_clip_dps #DATPens()
                         #grouped_clip_dps.append(last_clip_dps)
-                        #grouped_clip_dps.pens = last_clip_dps.pens
+                        #grouped_clip_dps._pens = last_clip_dps._pens
                         grouped_clip_dps.append(clip_dps)
                         #re_grouped_line[-1] = grouped_clip_dps
                         #last_clip_dps = grouped_clip_dps
@@ -532,9 +386,9 @@ class ClipGroup(Timeable):
             re_grouped.append(re_grouped_line)
         
         pens = re_grouped
-        for pens in pens.pens:
+        for pens in pens._pens:
             if removeOverlap:
-                for pen in pens.pens:
+                for pen in pens._pens:
                     pen.removeOverlap()
             group_pens.append(pens)
         
@@ -543,7 +397,7 @@ class ClipGroup(Timeable):
                 #pen.f(0)
                 pass
             if clip.blank:
-                pen.pens = []
+                pen._pens = []
         
         for line in group_pens:
             line.reversePens() # line top-to-bottom
@@ -556,7 +410,7 @@ class ClipGroup(Timeable):
         return group_pens
     
     def iterate_clip_pens(self, pens):
-        if hasattr(pens, "pens"):
+        if hasattr(pens, "_pens"):
             for pen in pens:
                 if hasattr(pen, "data"):
                     if pen.data.get("clip"):
@@ -567,16 +421,16 @@ class ClipGroup(Timeable):
     def remove_futures(self, pens):
         for clip, pen in self.iterate_clip_pens(pens):
             if clip.position > 0:
-                pen.pens = []
+                pen._pens = []
     
     def iterate_pens(self, pens, copy=True):
         for idx, line in enumerate(self.lines()):
             _pens = pens[idx]
             for cidx, clip in enumerate(line):
                 if copy:
-                    p = _pens.pens[cidx].copy()
+                    p = _pens._pens[cidx].copy()
                 else:
-                    p = _pens.pens[cidx]
+                    p = _pens._pens[cidx]
                 yield idx, clip, p
     
     def __repr__(self):
@@ -618,6 +472,27 @@ class ClipTrack():
             if clip.start <= fi and fi < clip.end:
                 return clip
     
+    def now(self, fi, text):
+        for idx, clip in enumerate(self.clips):
+            clip:Clip
+            if clip.text == text and clip.start <= fi and fi < clip.end:
+                return clip
+        return False
+
+    def now_or_past(self, fi, text):
+        for idx, clip in enumerate(self.clips):
+            clip:Clip
+            if clip.text == text and clip.start <= fi:
+                return clip
+        return False
+    
+    def over(self, fi, text):
+        for idx, clip in enumerate(self.clips):
+            clip:Clip
+            if clip.text == text and fi >= clip.end:
+                return True
+        return False
+    
     def duration(self):
         duration = 0
         for c in self.clips:
@@ -633,6 +508,47 @@ class Sequence(Timeline):
         self.workarea_track = workarea_track
         super().__init__(duration, fps, storyboard, tracks)
     
+    def find_symbol(self, symbol):
+        start = -1
+        end = -1
+        for t in self.tracks:
+            for c in t.clips:
+                c:Clip
+                if c.symbol and c.symbol == symbol:
+                    if c.symbol_position == -2:
+                        start = c.start
+                        end = c.end
+                    elif c.symbol_position == -1:
+                        start = c.start
+                    elif c.symbol_position == +1:
+                        end = c.end
+        return start, end
+    
+    def retime_for_symbol(self, symbol):
+        start, end = self.find_symbol(symbol)
+        if start == -1 or end == -1:
+            raise Exception("No symbol found")
+        
+        duration = end - start
+        self.start = 0
+        self.end = duration
+
+        # TODO storyboard?
+        for idx, frame in enumerate(self.storyboard):
+            self.storyboard[idx] = frame - start
+
+        for t in self.tracks:
+            t:ClipTrack
+            for c in t.clips:
+                c:Clip
+                c.start = c.start - start
+                c.end = c.end - start
+            
+            for cg in t.clip_groups:
+                cg.retime()
+        
+        return self
+    
     def trackClipGroupForFrame(self, track_idx, frame_idx, styles=[], check_end=True):
         for gidx, group in enumerate(self[track_idx].clip_groups):
             group:ClipGroup
